@@ -1,6 +1,11 @@
-from zeep.xsd.builtins import *  # noqa
+"""
+    zeep.xsd
+    --------
+
+"""
+from zeep.xsd.const import Nil, SkipValue  # noqa
 from zeep.xsd.elements import *  # noqa
-from zeep.xsd.types import *  # noqa
-from zeep.xsd.valueobjects import *  # noqa
 from zeep.xsd.schema import Schema  # noqa
-from zeep.xsd.indicators import *  # noqa
+from zeep.xsd.types import *  # noqa
+from zeep.xsd.types.builtins import *  # noqa
+from zeep.xsd.valueobjects import *  # noqa
