@@ -8,8 +8,8 @@ def test_main_no_args(monkeypatch):
     def mock_init(self, *args, **kwargs):
         self.wsdl = stub(dump=lambda: None)
 
-    monkeypatch.setattr(client.Client, '__init__', mock_init)
-    args = __main__.parse_arguments(['foo.wsdl'])
+    monkeypatch.setattr(client.Client, "__init__", mock_init)
+    args = __main__.parse_arguments(["foo.wsdl"])
     __main__.main(args)
 
 
@@ -17,15 +17,16 @@ def test_main_extract_auth(monkeypatch):
     def mock_init(self, *args, **kwargs):
         self.wsdl = stub(dump=lambda: None)
 
-    monkeypatch.setattr(client.Client, '__init__', mock_init)
+    monkeypatch.setattr(client.Client, "__init__", mock_init)
 
-    with patch.object(__main__, 'Transport', autospec=True) as mock_transport:
+    with patch.object(__main__, "Transport", autospec=True) as mock_transport:
         args = __main__.parse_arguments(
-            ['http://user:secret@tests.python-zeep.org/foo.wsdl'])
+            ["http://user:secret@tests.python-zeep.org/foo.wsdl"]
+        )
 
         __main__.main(args)
 
         assert mock_transport.call_count == 1
 
         args, kwargs = mock_transport.call_args
-        assert kwargs['session'].auth == ('user', 'secret')
+        assert kwargs["session"].auth == ("user", "secret")
